package ovh.herisson.Clyde.EndPoints.Inscription;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.CourseRepository;
import ovh.herisson.Clyde.Repositories.Inscription.ExemptionsRequestRepository;
import ovh.herisson.Clyde.Repositories.Inscription.ScholarshipRequestRepository;
import ovh.herisson.Clyde.Repositories.Inscription.UnregisterRequestRepository;
import ovh.herisson.Clyde.Repositories.UserRepository;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Tables.*;
import ovh.herisson.Clyde.Tables.Inscription.ExemptionsRequest;
import ovh.herisson.Clyde.Tables.Inscription.ScholarshipRequest;
import ovh.herisson.Clyde.Tables.Inscription.UnregisterRequest;

import java.util.ArrayList;
import java.util.Date;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class RequestsController {

    public final ExemptionsRequestRepository err;
    public final ScholarshipRequestRepository srr;
    public final UserRepository userRepository;
    public final AuthenticatorService authServ;
    public final UnregisterRequestRepository unregisterRequestRepository;
    public final CourseRepository courseRepository;


    public RequestsController(ExemptionsRequestRepository err, ScholarshipRequestRepository srr, UserRepository userRepository, AuthenticatorService authServ, UnregisterRequestRepository unregisterRequestRepository, CourseRepository courseRepository) {
        this.err = err;
        this.srr = srr;
        this.userRepository = userRepository;
        this.authServ = authServ;
        this.unregisterRequestRepository = unregisterRequestRepository;
        this.courseRepository = courseRepository;
    }

    @PostMapping(value="/exemptionreq")
    public ResponseEntity<String> createExemptionReq(@RequestBody Map<String, Object> exemptionsRequestInfo){
        User user = userRepository.findById((Integer) exemptionsRequestInfo.get("userRegNo"));
        Course course = courseRepository.findById((Integer) exemptionsRequestInfo.get("courseId"));

        ExemptionsRequest exemptionsRequest = new ExemptionsRequest(user, course, (String) exemptionsRequestInfo.get("justifDocument"), RequestState.Pending, new Date());

        err.save(exemptionsRequest);

        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @PostMapping(value="/scholarshipreq")
    public ResponseEntity<String> createScholarshipReq(@RequestBody Map<String, Object> scholarshipRequestInfo){
        User user = userRepository.findById((Integer)scholarshipRequestInfo.get("userId"));
        ScholarshipRequest toCreate = new ScholarshipRequest(user, RequestState.Pending, 0, new Date(), (String) scholarshipRequestInfo.get("taxDocUrl"), (String) scholarshipRequestInfo.get("residencyDocUrl"));

        srr.save(toCreate);

        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    //Get all the exemptions Request
    @GetMapping(value = "/exemptionsreq")
    public ResponseEntity<ArrayList<ExemptionsRequest>> getAllExemptionsRequests(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<ExemptionsRequest> toReturn = new ArrayList<>();

        err.findAll().forEach(toReturn::add);

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    //Get all the scholarships requests
    @GetMapping(value = "/scholarshipreq")
    public ResponseEntity<ArrayList<ScholarshipRequest>> getAllScholarshipRequests(@RequestHeader("Authorization") String token){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary,Role.InscriptionService},token))
            return new UnauthorizedResponse<>(null);

        ArrayList<ScholarshipRequest> toReturn = new ArrayList<>();

        srr.findAll().forEach(toReturn::add);

        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @PostMapping(value = "/unregister")
    public ResponseEntity<String> postUnregReq(@RequestBody Map<String,Object> uninscr){
        User u = userRepository.findById((int) uninscr.get("userId"));
        UnregisterRequest ur = new UnregisterRequest(RequestState.Pending, (String) uninscr.get("reason"), new Date(), u.getRegNo(), u.getFirstName(), u.getLastName(), u.getEmail());
        unregisterRequestRepository.save(ur);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PatchMapping(value = "/scholarshipreq/")
    public ResponseEntity<String> editScholReq(@RequestBody Map<String,Object> infos){
        ScholarshipRequest scholarshipRequest = srr.findById((Integer) infos.get("id"));

        if (infos.get("state").equals("Accepted")){
            scholarshipRequest.setState(RequestState.Accepted);
            scholarshipRequest.setAmount((int) infos.get("amount"));
        }else{
            scholarshipRequest.setState(RequestState.Refused);
        }

        srr.save(scholarshipRequest);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping(value = "/scholarshipreq/{id}")
    public ResponseEntity<ScholarshipRequest> getScholReqbyId(@PathVariable long id){
        ScholarshipRequest toReturn = srr.findById(id);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }

    @GetMapping(value = "/unregister")
    public ResponseEntity<ArrayList<UnregisterRequest>> getAllUnregReq(){
        ArrayList<UnregisterRequest> toReturn = new ArrayList<>();
        unregisterRequestRepository.findAll().forEach(toReturn::add);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }
}
